﻿#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file DetectRPSWinner.py
 @brief Detect Janken(Rock Paper Scissors) Winner
 @date $Date$


"""
import sys
import time
sys.path.append(".")

# Import RTM module
import RTC
import OpenRTM_aist


# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
detectrpswinner_spec = ["implementation_id", "DetectRPSWinner", 
		 "type_name",         "DetectRPSWinner", 
		 "description",       "Detect Janken(Rock Paper Scissors) Winner", 
		 "version",           "1.0.0", 
		 "vendor",            "Yuya Tatsuta", 
		 "category",          "Category", 
		 "activity_type",     "STATIC", 
		 "max_instance",      "1", 
		 "language",          "Python", 
		 "lang_type",         "SCRIPT",
		 ""]
# </rtc-template>

##
# @class DetectRPSWinner
# @brief Detect Janken(Rock Paper Scissors) Winner
# 
# 
class DetectRPSWinner(OpenRTM_aist.DataFlowComponentBase):
	
	##
	# @brief constructor
	# @param manager Maneger Object
	# 
	def __init__(self, manager):
		OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

		self._d_PlayerHand = OpenRTM_aist.instantiateDataType(RTC.TimedLong)
		"""
		1: Rock
		2: Paper
		3: Scissors
		"""
		self._PlayerHandIn = OpenRTM_aist.InPort("PlayerHand", self._d_PlayerHand)
		self._d_ComputerHand = OpenRTM_aist.instantiateDataType(RTC.TimedLong)
		"""
		1: Rock
		2: Paper
		3: Scissors
		"""
		self._ComputerHandIn = OpenRTM_aist.InPort("ComputerHand", self._d_ComputerHand)
		self._d_Result = OpenRTM_aist.instantiateDataType(RTC.TimedLong)
		"""
		1: Player win.
		2: Player lose.
		"""
		self._ResultOut = OpenRTM_aist.OutPort("Result", self._d_Result)


		


		# initialize of configuration-data.
		# <rtc-template block="init_conf_param">
		
		# </rtc-template>


		 
	##
	#
	# The initialize action (on CREATED->ALIVE transition)
	# formaer rtc_init_entry() 
	# 
	# @return RTC::ReturnCode_t
	# 
	#
	def onInitialize(self):
		# Bind variables and configuration variable
		
		# Set InPort buffers
		self.addInPort("PlayerHand",self._PlayerHandIn)
		self.addInPort("ComputerHand",self._ComputerHandIn)
		
		# Set OutPort buffers
		self.addOutPort("Result",self._ResultOut)
		
		# Set service provider to Ports
		
		# Set service consumers to Ports
		
		# Set CORBA Service Ports
		
		return RTC.RTC_OK
	
	###
	## 
	## The finalize action (on ALIVE->END transition)
	## formaer rtc_exiting_entry()
	## 
	## @return RTC::ReturnCode_t
	#
	## 
	#def onFinalize(self):
	#
	#	return RTC.RTC_OK
	
	###
	##
	## The startup action when ExecutionContext startup
	## former rtc_starting_entry()
	## 
	## @param ec_id target ExecutionContext Id
	##
	## @return RTC::ReturnCode_t
	##
	##
	#def onStartup(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	###
	##
	## The shutdown action when ExecutionContext stop
	## former rtc_stopping_entry()
	##
	## @param ec_id target ExecutionContext Id
	##
	## @return RTC::ReturnCode_t
	##
	##
	#def onShutdown(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	###
	##
	## The activated action (Active state entry action)
	## former rtc_active_entry()
	##
	## @param ec_id target ExecutionContext Id
	## 
	## @return RTC::ReturnCode_t
	##
	##
	#def onActivated(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	###
	##
	## The deactivated action (Active state exit action)
	## former rtc_active_exit()
	##
	## @param ec_id target ExecutionContext Id
	##
	## @return RTC::ReturnCode_t
	##
	##
	#def onDeactivated(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	##
	#
	# The execution action that is invoked periodically
	# former rtc_active_do()
	#
	# @param ec_id target ExecutionContext Id
	#
	# @return RTC::ReturnCode_t
	#
	#
	def onExecute(self, ec_id):
		if self._PlayerHandIn.isNew():
			player_hand = self._PlayerHandIn.read().data
			computer_hand = self._ComputerHandIn.read().data
			result = 0
			print(player_hand, computer_hand)
		
			# detect logic
			if(player_hand == 0 or computer_hand == 0):
				result = 0
			elif(player_hand == computer_hand):
				result = 3
			elif(player_hand - computer_hand > 0):
				result = player_hand - computer_hand
			else:
				result = 3 - (computer_hand - player_hand)
		
			self._d_Result.data = result
			self._ResultOut.write()
			print(player_hand, computer_hand, result)
		return RTC.RTC_OK
	
	###
	##
	## The aborting action when main logic error occurred.
	## former rtc_aborting_entry()
	##
	## @param ec_id target ExecutionContext Id
	##
	## @return RTC::ReturnCode_t
	##
	##
	#def onAborting(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	###
	##
	## The error action in ERROR state
	## former rtc_error_do()
	##
	## @param ec_id target ExecutionContext Id
	##
	## @return RTC::ReturnCode_t
	##
	##
	#def onError(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	###
	##
	## The reset action that is invoked resetting
	## This is same but different the former rtc_init_entry()
	##
	## @param ec_id target ExecutionContext Id
	##
	## @return RTC::ReturnCode_t
	##
	##
	#def onReset(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	###
	##
	## The state update action that is invoked after onExecute() action
	## no corresponding operation exists in OpenRTm-aist-0.2.0
	##
	## @param ec_id target ExecutionContext Id
	##
	## @return RTC::ReturnCode_t
	##

	##
	#def onStateUpdate(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	###
	##
	## The action that is invoked when execution context's rate is changed
	## no corresponding operation exists in OpenRTm-aist-0.2.0
	##
	## @param ec_id target ExecutionContext Id
	##
	## @return RTC::ReturnCode_t
	##
	##
	#def onRateChanged(self, ec_id):
	#
	#	return RTC.RTC_OK
	



def DetectRPSWinnerInit(manager):
    profile = OpenRTM_aist.Properties(defaults_str=detectrpswinner_spec)
    manager.registerFactory(profile,
                            DetectRPSWinner,
                            OpenRTM_aist.Delete)

def MyModuleInit(manager):
    DetectRPSWinnerInit(manager)

    # Create a component
    comp = manager.createComponent("DetectRPSWinner")

def main():
	mgr = OpenRTM_aist.Manager.init(sys.argv)
	mgr.setModuleInitProc(MyModuleInit)
	mgr.activateManager()
	mgr.runManager()

if __name__ == "__main__":
	main()

